local M = {}

local helper = require('scenario/scenariohelper')
local logTag = 'Traffic_Battle_1'

local finalWaypointName = 'Wpoint27'
local playerInstance = 'scenario_player0'
local running = false
local playerWon = false
local aiWon = false
local noOfLaps = 1
local lapsCompleted = {}
local raceLap = {"RaceWPoint1", "RaceWPoint2", "RaceWPoint3", "RaceWPoint4", "RaceWPoint5", "RaceWPoint6", "RaceWPoint7", "RaceWPoint8", "RaceWPoint8", "Wpoint17", "Wpoint16", "Wpoint15", "RaceWPoint1"}
local cityLap = {"wp_town11", "wp_town10", "wpTown_9", "wpTown_10", "TownWPoint1", "TownWPoint2", "wp_town11"}

local function reset()
  lapsCompleted['scenario_player0'] = 0
--  lapsCompleted['scenario_opponent1'] = 0
--  lapsCompleted['scenario_opponent2'] = 0
--  lapsCompleted['scenario_opponent3'] = 0

  running = false
  playerWon = false
  aiWon = false
end

local function fail(reason)
  scenario_scenarios.finish({failed = reason})
  reset()
end

local function success(reason)
  scenario_scenarios.finish({msg = reason})
  reset()
end

local function onRaceStart()

  scenario_scenarios.trackVehicleMovementAfterDamage(playerInstance, {waitTimerLimit=10})
  running = true
end

local function setupVehicleAI(minSpeed, maxSpeed, minAggression, maxAggression, vehicleName, waypoints, routeSpeedMode, driveInLane, lapCount)
  local r = math.random (minSpeed, maxSpeed)
  local r2 = math.random (minAggression, maxAggression)
  local arg = {vehicleName=vehicleName, waypoints=waypoints, aggression = r2, routeSpeed = r, routeSpeedMode=routeSpeedMode, driveInLane=driveInLane, resetLearning = true, lapCount=lapCount}
  helper.setAiPath(arg)
end

local function onCountdownStarted()
  reset()
  setupVehicleAI(20, 40, 0.7, 1.4, "AI_Nr_Home", {"Wpoint6", "Bridge6_A"}, "limit", "on", 1)
  setupVehicleAI(15, 40, 0.7, 1, "Bus", {"Wpoint1"}, "limit", "on", 1)
end

local function onRaceWaypointReached(data)
  --log('I', logTag,'onRaceWaypointReached called ')
  if data.waypointName == 'Wpoint6' then
	TorqueScript.eval([[
            AI_Nr_Home.position = "368.57 -330.632 149.643";
            AI_Nr_Home.rotation = "0.000395718 0.000345267 -1 82.21";
        ]])
	setupVehicleAI(20, 40, 0.7, 1.4, "AI_Nr_Home", {"countryside_road_1"}, "limit", "on", 1)
	elseif data.waypointName == 'Wpoint14' then
    setupVehicleAI(80, 100, 1, 1.4, "Race1", raceLap, "limit", "off", 999)
    setupVehicleAI(80, 100, 1, 1.4, "Race2", raceLap, "limit", "off", 999)
	elseif data.waypointName == 'Wpoint17' then
		TorqueScript.eval([[
			Race1.position = "-917.361 386.039 155.708";
			Race1.rotation = "1 0 0 2.56536";
		]])
		setupVehicleAI(10, 11, 0.9, 1.1, "Race1", cityLap, "limit", "on", 999)
	elseif data.waypointName == 'Wpoint18' then
		TorqueScript.eval([[
	        AI_Nr_Home.position = "-917.361 386.039 155.708";
            AI_Nr_Home.rotation = "1 0 0 2.56536";
        ]])
		setupVehicleAI(10, 11, 0.9, 1.1, "AI_Nr_Home", cityLap, "limit", "on", 999)
	elseif data.waypointName == 'Wpoint19' then
		TorqueScript.eval([[
			Race2.position = "-917.361 386.039 155.708";
			Race2.rotation = "1 0 0 2.56536";
		]])
		setupVehicleAI(10, 11, 0.9, 1.1, "Race2", cityLap, "limit", "on", 999)
  elseif data.waypointName == 'Wpoint21' then
	TorqueScript.eval([[
            Bus.position = "-917.361 386.039 155.708";
            Bus.rotation = "1 0 0 2.56536";
        ]])
		setupVehicleAI(10, 13, 0.8, 0.9, "Bus", cityLap, "limit", "on", 999)
	end
  if data.waypointName == finalWaypointName then
    lapsCompleted[data.vehicleName] = lapsCompleted[data.vehicleName] + 1
    if lapsCompleted[data.vehicleName] == noOfLaps then
      if data.vehicleName == playerInstance and not aiWon then
        playerWon = true
      elseif not playerWon then
        aiWon = true
      end
    end
  end
end

local function onRaceResult()
  if playerWon then
    success("Boom")
  else
    fail("No boom")
  end
end

M.onRaceStart = onRaceStart
M.onCountdownStarted = onCountdownStarted
M.onRaceWaypointReached = onRaceWaypointReached
M.onRaceResult = onRaceResult
return M